/*
 * @Author: thy
 * @Date: 2023-10-26 14:56:41
 * @LastEditors: thy
 * @LastEditTime: 2023-11-29 17:54:47
 * @Description: file content
 * @FilePath: \dsk-operate-ui\src\utils\postMessageBridge\bridge\index.js
 */

import EventCenter from "./eventCenter";
import action from "@/utils/postMessageBridge/action";
import { normalToPromise } from "@/utils/postMessageBridge/tools";


class PostMessageBridge {
  // 当前系统
  _currentSystem = null;
  // 当前系统域
  _currentOriginUrl = null;
  // 目标系统
  _targetSystem = null;
  // 目标域
  _targetOriginUrl = null;
  // 事件调度中心
  _eventCenter = null;
  // 触发的行为
  _actionMap = null;
  // 当前vue实例
  _vueIns = null;
  /**
   * 
   * @param {{ 
   * currentSystem : Window || undefined;
   * currentOriginUrl : string;
   * targetSystem : Window || undefined;
   * targetOriginUrl : string || undefined;
   * vueIns : any;
   * }} options 
   */
  constructor(options) {
    return new Promise(async (resolve, reject) => {
      try {
        this._currentSystem = options.currentSystem;
        this._currentOriginUrl = options.currentOriginUrl || location.origin;
        this._targetSystem = options.targetSystem;
        this._targetOriginUrl = options.targetOriginUrl || "*";
        this._vueIns = options.vueIns || null;
        await this.init();
        resolve(this);
      } catch (error) {
        reject(error);
      }
    });
  }

  /**
   * 通信初始化
   */
  async init() {
    try {
      this._eventCenter = new EventCenter();
      this._actionMap = action;
      this._currentSystem.addEventListener("message", this.messageListener, false);
    } catch (error) {
      console.log(error);
    }
  }

  /**
   * 触发message 时执行的函数
   * @param {MessageEvent} event 
   * @returns 
   */
  messageListener(event) {
    new Promise(async (resolve, reject) => {
      try {
        const { data, origin, source } = event;
        if (origin !== this._currentOriginUrl) return;
        const { id, action, methodName, params } = data;
        if (!id) return;
        // promise化
        const result = await normalToPromise(this._actionMap[action][methodName])(params);
        resolve(result);
      } catch (error) {
        console.log(error);
        reject(error);
      }
    });
  };
}

export default PostMessageBridge;